/*
 *  Fenix - Videogame compiler/interpreter
 *  Current release       : FENIX - PROJECT 1.0 - R 0.84
 *  Last stable release   :
 *  Project documentation : http://fenix.divsite.net
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Copyright  1999 Jos Luis Cebrin Page
 *  Copyright  2002 Fenix Team
 *
 *  Update for Dreamcast by SWAT
 *
 */

/*
 * FILE        : main.c
 * DESCRIPTION : Main entry point for FXI
 *
 * HISTORY:	0.81 - Removed -w option
 *
 */

/*
 * INCLUDES
 */

#ifdef TARGET_BEOS
#include <posix/assert.h>
#else
#include <assert.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "fxi.h"
#include "font.h"
#include "dcb.h"

#include "GUI/button.h"
#include "GUI/listbox.h"
#include "GUI/msgbox.h"
#include "GUI/scrollbar.h"
#include "GUI/drawutil.h"

/* os versions */
#ifdef WIN32
	#define OS_ID 0
#endif
#ifdef TARGET_Linux
	#define OS_ID 1
#endif
#ifdef TARGET_BEOS
	#define OS_ID 2
#endif
#ifdef TARGET_MAC
	#define OS_ID 3
#endif
#ifdef TARGET_GP32
	#define OS_ID 4
#endif
#ifdef TARGET_DC
	#define OS_ID 5
	#include <kos.h>
#endif
#ifdef TARGET_BSD
	#define OS_ID 6
#endif
#ifdef WIN32
#include <windows.h>
#endif

#define DCB_MAGIC	"DCB Stub\x1A\x0D\x0A"
/*
 *  GLOBAL VARIABLES
 */

int debug	 = 1;		/* 1 if running in debug mode					 */
int fxi		 = 1;		/* 1 only if this is an standalone interpreter   */
int embedded = 0;		/* 1 only if this is a stub with an embedded DCB */



extern int full_screen, double_buffer, hardware_scr, enable_filtering;
extern SDL_Surface * screen;
extern char *apptitle;

static SDL_Surface * fscreen;


/*
 *  FUNCTION : do_exit
 *
 *  Exits from the program cleanly ending operations
 *
 *  PARAMS:
 *      INT n: ERROR LEVEL to return to OS
 *
 *  RETURN VALUE:
 *      No value
 *
 */

void
do_exit(int n)
{
    int i;

	if (keytab_initialized) keytab_free() ;

	for (i=0;i<MAX_JOYS;i++)
	    if (_joysticks[i]!=NULL) SDL_JoystickClose(_joysticks[i]) ;

	SDL_Quit() ;
	exit(n) ;
}

/*
 *  FUNCTION : main
 *
 *  Main function for FXI
 *
 *  PARAMS:
 *      INT n: ERROR LEVEL to return to OS
 *
 *  RETURN VALUE:
 *      No value
 *
 */
 


int SetupGUI(SDL_Surface *surface)
{

    back_color = SDL_MapRGB(surface->format, 179, 179, 179);
	light_color = SDL_MapRGB(surface->format, 225, 225, 225);
	shadow_color = SDL_MapRGB(surface->format, 104, 104, 104);
	loadFont();

	return 1;
}




int RunFile(char *filename) {
    
	INSTANCE *			mainproc_running ;
	char				dcbname[256];
	char msg[1024];
	
	/* Init application title for windowed modes */
	
	gprof_init () ;
	string_init () ;
	init_c_type() ;
    
   	
	strcpy (dcbname, filename) ;
	apptitle = strdup(filename) ;



	if (!dcb_load(dcbname)) {
		 //gr_error ("%s: not exist or DCB version %d not compatible", filename, DCB_VERSION >> 8) ;
         fscreen = SDL_SetVideoMode (640, 480, 16, SDL_HWSURFACE);
	     //SetupGUI(fscreen);
         sprintf(msg, "Executing error!\nPossible reasons:\n\n-File \"%s\" is not exist.\n-Selected file is not DCB file.\n-DCB version %d not compatible.\n", filename, DCB_VERSION >> 8);
         messageBox(fscreen, msg); 
         return 0;
	}



	/* If the dcb is not in debug mode, switch off the profiler */

	if (dcb.data.NID == 0)
	{
		gprof_toggle();
		debug = 0;
	}

	/* Initialization (modules needed after dcb_load) */

	fnc_init();
	grlib_init () ;
	sysproc_init () ;
	gr_font_systemfont (default_font);
	
	GLODWORD(FXI_OS) = OS_ID ;
	
	if (mainproc)
	{
		mainproc_running = instance_new (mainproc, 0) ;
		instance_go_all () ;
	}
    
return 1; 
}



extern export_sym_t fnx_symtab[];
static symtab_handler_t st_fnx = {
	{ "sym/fenix",
	  0,
	  0x00010000,
	  0,
	  NMMGR_TYPE_SYMTAB,
	  NMMGR_LIST_INIT },
	fnx_symtab
};

extern uint8 romdisk[];
KOS_INIT_ROMDISK(romdisk);

int main (int argc, char **argv)
{
	char				*filename;
	INSTANCE *			mainproc_running ;
	file *				fp = NULL;
	full_screen = 1; 
    //double_buffer = 1; 
    hardware_scr = 1; 
    //enable_filtering = 1;
    SDL_Surface * fscreen;
    int run = 0;
    char msg[1024];


	/* Init RAND generator */

	srand (time(NULL)) ;
	

	/* Init SDL */
	
         //SDL_DC_ShowAskHz(SDL_FALSE);
         SDL_DC_SetVideoDriver(SDL_DC_DIRECT_VIDEO);

	if ( SDL_Init (SDL_INIT_VIDEO | SDL_INIT_JOYSTICK | SDL_INIT_CDROM) < 0 )
	{
		printf ("SDL Init Error: %s\n", SDL_GetError()) ;
		do_exit(1) ;
	}
	
	//net_init();	
	
	SDL_JoystickOpen(0);
    SDL_JoystickEventState(SDL_ENABLE);
    SDL_ShowCursor(SDL_ENABLE);
    SDL_DC_EmulateMouse(SDL_TRUE);
    SDL_DC_EmulateKeyboard(SDL_TRUE);
	
	if (nmmgr_handler_add(&st_fnx.nmmgr) < 0) {                                
		printf ("Can't init exports\n") ;
        //return 0;
    }

/*	
	SDL_DC_MapKey(0, SDL_DC_START, SDLK_RETURN);
	SDL_DC_MapKey(0, SDL_DC_A, SDLK_s);
	SDL_DC_MapKey(0, SDL_DC_B, SDLK_d);
	SDL_DC_MapKey(0, SDL_DC_X, SDLK_a);
	SDL_DC_MapKey(0, SDL_DC_Y, SDLK_w);
	SDL_DC_MapKey(0, SDL_DC_LEFT, SDLK_LEFT);
	SDL_DC_MapKey(0, SDL_DC_RIGHT, SDLK_RIGHT);
	SDL_DC_MapKey(0, SDL_DC_UP, SDLK_UP);
	SDL_DC_MapKey(0, SDL_DC_DOWN, SDLK_DOWN);*/


    fscreen = SDL_SetVideoMode (640, 480, 16, SDL_HWSURFACE);
	SetupGUI(fscreen);
    gr_con_printf ("15" FXI_VERSION);
    printf(FXI_VERSION "\n");

    while(1) {
             drawText(fscreen, 140, charHeight()-4, " Fenix engine v0.93 build 3 for Dreamcast by SWAT ");
             filename = openFile(fscreen, ""); 
             if(!filename) {
                           sprintf(msg, "File not selected\n");
                           messageBox(fscreen, msg);
                           continue;
                           }
             //sprintf(msg, "File for executing: %s\n", filename);
             //messageBox(fscreen, msg);
             RunFile(filename);
    }
	
	do_exit(0);
	return 0;
}

